"""Version Command"""

import logging
import click
import pika

logger = logging.getLogger(__name__)


@click.command(
    "check",
    help="Checks the connection of a RabbitMQ server.",
    short_help="Check connection.",
)
@click.option(
    "--server",
    "server",
    required=True,
    type=str,
    help="URL of the RabbitMQ server",
)
@click.option(
    "--virtual-host",
    default="/",
    type=str,
    show_default=True,
    help="The virtual host to use",
)
@click.option(
    "--ssl",
    required=False,
    default=False,
    type=bool,
    show_default=True,
    help="Enable SSL",
)
@click.option(
    "--port",
    required=False,
    default=5672,
    type=int,
    show_default=True,
    help="Server port",
)
@click.option(
    "--username",
    required=False,
    default="guest",
    type=str,
    show_default=True,
    help="User account",
)
@click.option(
    "--password",
    required=False,
    default="guest",
    type=str,
    show_default=True,
    hide_input=True,
    help="User password",
)
@click.help_option("-h", "--help")
def cli(server, virtual_host, ssl, port, username, password):
    """Entry point for command."""
    credentials = pika.PlainCredentials(username, password)
    parameters = pika.ConnectionParameters(
        host=server,
        port=port,
        virtual_host=virtual_host,
        credentials=credentials,
    )

    try:
        connection = pika.BlockingConnection(parameters)
        if connection.is_open:
            print("OK")
            connection.close()
            exit(0)
    except Exception as error:
        print(f"Error: {error.__class__.__name__}")
        exit(1)



"""Entry point for CLI."""

import os
import click
import logging

from pathlib import Path

CMD_FOLDER = str(Path(os.path.dirname(__file__), "commands").resolve())
CMD_PREFIX = "cmd_"

logger = logging.getLogger(__name__)


class RabbotCLI(click.MultiCommand):
    """
    Multi-command class for Rabbot. Handles gathering subcommands.
    """

    def list_commands(self, ctx):
        """
        Creates a list of commands by reading in the files in the CLI's
        'commands' directory.

        :param ctx:
        :return: List of strings representing the names of commands
        """
        commands = []
        for filename in os.listdir(CMD_FOLDER):
            if filename.endswith(".py") and filename.startswith(CMD_PREFIX):
                commands.append(filename[4:-3])
        commands.sort()
        return commands

    def get_command(self, ctx, name):
        """
        Modifies Click's subcommands by importing each 'cmd_' file as a
        module, adding them to the UI, and making them available
        to the caller.

        :param ctx:
        :param name: name of the command
        :return: Returns the modified cli object containing the available cmds.
        """

        try:
            cmd_base = f"rabbot.commands.{CMD_PREFIX}"
            mod = __import__(cmd_base + name, None, None, ["cli"])
            if hasattr(mod, "subcommand"):
                return getattr(mod, "subcommand")
        except ImportError:
            return
        return mod.cli


@click.command(
    cls=RabbotCLI,
    epilog='See "COMMAND -h" to read about a specific subcommand',
    short_help="%(prog)s [-h] COMMAND [args]",
)
@click.help_option("-h", "--help")
def cli():
    """
    Rabbot is a utility for testing connections to RabbitMQ servers.
    """
    pass


if __name__ == "__main__":
    cli()



"""Version Command"""

import sys

import click

from rabbot.__version__ import __version__


class Version:
    """Contains all version details."""
    def __init__(self, version=__version__):
        super(Version, self).__init__()
        self._version = version

    def about(self):
        """Returns a dict containing the version details"""
        about = {
            '__version__': self._version
        }

        return about


@click.command(
    'version',
    help='Shows the current version of the CLI and exits.',
    short_help='Shows the current version of the CLI.'
)
@click.help_option('-h', '--help')
def cli():
    """Entry point for command."""
    version = Version()
    click.echo(
        '{0} {1}'.format('Rabbot', version.about().get('__version__'))
    )
    sys.exit(0)
